//
//  FacebookOAuthHelper.m
//  FacebookOAuth
//
//  Created by Reid Simonton on 2/7/14.
//  Copyright (c) 2014 Temboo, Inc. All rights reserved.
//
#include <stdlib.h> // arc4random_uniform requirement

#import "FacebookOAuthHelper.h"
#import "TMBTembooSession.h"
#import "TMBFacebook.h"

@interface FacebookOAuthHelper ()
@property (nonatomic, readwrite, strong) NSString *authorizationUrl;
@property (nonatomic, readwrite, strong) NSString *stateToken;
@property (nonatomic, readwrite, strong) TMBTembooSession *session;
@property (nonatomic, readwrite, strong) NSString *facebookAccessToken;
@property (nonatomic, readwrite, strong) id oauthFinalizedDelegate;
@property (nonatomic, readwrite, strong) id loginUrlDelegate;
@end

@implementation FacebookOAuthHelper

@synthesize authorizationUrl;
@synthesize facebookAccessToken;
@synthesize loginUrlDelegate;
@synthesize oauthFinalizedDelegate;
@synthesize session;
@synthesize stateToken;

// Provide your Facebook App ID and App Secret.
static NSString* FACEBOOK_APP_ID = @"YOUR_FACEBOOK_APP_ID";
static NSString* FACEBOOK_APP_SECRET = @"YOUR_FACEBOOK_APP_SECRET";

// Replace with your Temboo credentials.
static NSString* TEMBOO_ACCOUNT_NAME = @"YOUR_ACCOUNT_NAME";
static NSString* TEMBOO_APP_KEY_NAME = @"YOUR_APP_KEY_NAME";
static NSString* TEMBOO_APP_KEY_VALUE = @"YOUR_APP_KEY_VALUE";

-(id)init {
    if(self = [super init]){
        self.session = [[TMBTembooSession alloc] initWithAccount:TEMBOO_ACCOUNT_NAME
                                                  appKeyName:TEMBOO_APP_KEY_NAME
                                                  andAppKeyValue:TEMBOO_APP_KEY_VALUE];
        
        // Generate a cryptographically-strong random token
        self.stateToken = [NSString stringWithFormat:@"facebook-%u", arc4random_uniform(NSIntegerMax)];
        NSLog(@"Set state token: %@", self.stateToken);
    }
    
    return self;
}

-(void)choreographyDidFailWithError:(NSError *)error {
    NSLog(@"%s - An error occurred: %@", __PRETTY_FUNCTION__, error);
}

-(void)choreographyDidFinishExecuting:(id)result {
    if([result isKindOfClass:[TMBFacebook_OAuth_InitializeOAuth_ResultSet class]]){
        self.authorizationUrl = [result getAuthorizationURL];
        [self.loginUrlDelegate authorizationUrlDidLoad:self.authorizationUrl];
    }
    else if([result isKindOfClass:[TMBFacebook_OAuth_FinalizeOAuth_ResultSet class]]){
        self.facebookAccessToken = [result getAccessToken];
        [self.oauthFinalizedDelegate oauthDidFinalize:self.facebookAccessToken];
    }
}

-(void)finalizeOAuthForDelegate:(id)delegate {
    self. oauthFinalizedDelegate = delegate;
    
    TMBFacebook_OAuth_FinalizeOAuth *choreo = [[TMBFacebook_OAuth_FinalizeOAuth alloc] initWithSession:self.session];
    TMBFacebook_OAuth_FinalizeOAuth_Inputs *inputs = [choreo newInputSet];
    
    [inputs setAppID:FACEBOOK_APP_ID];
    [inputs setAppSecret:FACEBOOK_APP_SECRET];
    [inputs setLongLivedToken:@"1"];
    [inputs setCallbackID:[NSString stringWithFormat:@"%@/%@", TEMBOO_ACCOUNT_NAME, self.stateToken]];
    
    [choreo executeWithInputs:inputs delegate:self];
}

// Executes Facebook.OAuth.InitializeOAuth in order to get an AuthorizationURL
-(void)getAuthorizationUrlWithForwardingURL:(NSString*)url delegate:(id)delegate {
    
    self.loginUrlDelegate = delegate;
    
    TMBFacebook_OAuth_InitializeOAuth *choreo = [[TMBFacebook_OAuth_InitializeOAuth alloc] initWithSession:self.session];
    TMBFacebook_OAuth_InitializeOAuth_Inputs *inputs = [choreo newInputSet];
    
    [inputs setAppID:FACEBOOK_APP_ID];
    [inputs setCustomCallbackID:self.stateToken];
    [inputs setForwardingURL:[NSString stringWithFormat:@"%@?state=%@", url, self.stateToken]];
    
    [choreo executeWithInputs:inputs delegate:self];
}

-(TMBTembooSession*)getUserSession {
    return self.session;
}

@end
