//
//  ViewController.m
//  FacebookOAuth
//
//  Created by Reid Simonton on 2/6/14.
//  Copyright (c) 2014 Temboo, Inc. All rights reserved.
//

#import "ViewController.h"
#import "FacebookOAuthHelper.h"
#import "HTTPServer.h"
#import "UserInfo.h"

@interface ViewController ()
@property (nonatomic, readwrite, strong) NSString *authorizationUrl;
@property (nonatomic, readwrite, strong) FacebookOAuthHelper *oauthHelper;
@end

@implementation ViewController

// Callback URI that Temboo will redirect to after successful authentication.
// Note that a random state parameter gets appended to this in the Initialize step.
static NSString* const FORWARDING_URL = @"http://localhost:12345";

@synthesize authorizationUrl;
@synthesize loginButton;
@synthesize oauthHelper;
@synthesize spinner;

-(void)authorizationUrlDidLoad:(NSString*)url {
    NSLog(@"Authorization URL loaded: %@", url);
    self.authorizationUrl = url;
    
    // Hide the spinner
    [self.spinner stopAnimating];
    
    // Update the button text
    NSString *title = @"Facebook Login";
    [self.loginButton setTitle:title forState:UIControlStateNormal];
    [self.loginButton setTitle:title forState:UIControlStateApplication];
    [self.loginButton setTitle:title forState:UIControlStateHighlighted];
    [self.loginButton setTitle:title forState:UIControlStateReserved];
    [self.loginButton setTitle:title forState:UIControlStateSelected];
    [self.loginButton setTitle:title forState:UIControlStateDisabled];
    
    // Make it clickable
    [self.loginButton setEnabled:YES];
}

/*! Pass the auth URL to the web view */
-(void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    NSLog(@"prepareForSegue: %@", segue.identifier);
    UserInfo *userInfo = [segue destinationViewController];
    userInfo.authorizationUrl = self.authorizationUrl;
    userInfo.oauthHelper = self.oauthHelper;
}

- (void)viewDidLoad {
    [super viewDidLoad];
	NSLog(@"Loading authorization URL");
    
    self.oauthHelper = [[FacebookOAuthHelper alloc] init];
    [self.oauthHelper getAuthorizationUrlWithForwardingURL:FORWARDING_URL delegate:self];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)loginButton:(id)sender {
    NSLog(@"Clicked login button");
}

@end
